#!/usr/bin/env python
import grp, pwd
import os

from resource_management import *
import subprocess
import os

class Httpfs_component(Script):
    def configure(self, env):
        import params
        Logger.info("Configure HttpFS service")
        try:
            grp.getgrnam(params.httpfs_group)
        except KeyError:
            Logger.info(format("Creating group '{params.httpfs_group}' for HttpFS"))
            Group(
                group_name      = params.httpfs_group,
                ignore_failures = params.ignore_groupsusers_create
            )
        try:
            pwd.getpwnam(params.httpfs_user)
        except KeyError:
            Logger.info(format("Creating user '{params.httpfs_user}' for HttpFS"))
            User(
                username        = params.httpfs_user,
                groups          = [ params.httpfs_group ],
                ignore_failures = params.ignore_groupsusers_create
            )
        Directory( params.httpfs_conf_dir,
            owner     = params.httpfs_user,
            group     = params.httpfs_group,
            create_parents = True,
            mode      = 0o755,
        )
        Directory( params.httpfs_log_dir,
            owner     = params.httpfs_user,
            group     = params.httpfs_group,
            create_parents = True,
            mode      = 0o755,
        )
        Logger.info( format("Creating {params.httpfs_conf_dir}/httpfs-env.sh config file"))
        File( format("{params.httpfs_conf_dir}/httpfs-env.sh"),
            content   = InlineTemplate(params.httpfs_env_template,
                httpfs_server_port       = params.httpfs_server_port,
                httpfs_server_admin_port = params.httpfs_server_admin_port,
                httpfs_log_dir           = params.httpfs_log_dir,
                httpfs_conf_dir           = params.httpfs_conf_dir
            ),
            owner     = params.httpfs_user,
            group     = params.httpfs_group,
            mode      = 0o644
        )
        Logger.info(format("Creating {params.httpfs_conf_dir}/httpfs-env.sh config file - DONE"))
        Logger.info(format("Creating {params.httpfs_conf_dir}/httpfs-site.xml config file"))
        XmlConfig("httpfs-site.xml",
            mode                     = 0o644,
            owner                    = params.httpfs_user,
            group                    = params.httpfs_group,
            conf_dir                 = params.httpfs_conf_dir,
            configurations           = params.config['configurations']['httpfs-site'],
            configuration_attributes = params.config['configuration_attributes']['httpfs-site']
        )
        Logger.info(format("Creating {params.httpfs_conf_dir}/httpfs-site.xml config file - DONE"))
        Logger.info(format("Creating {params.httpfs_conf_dir}/httpfs-log4j.properties config file"))
        File(format("{params.httpfs_conf_dir}/httpfs-log4j.properties"),
            mode  = 0o644,
            owner = params.httpfs_user,
            group = params.httpfs_group,
            content = params.httpfs_log4j_content
        )

        Logger.info(format("Creating {params.httpfs_conf_dir}/httpfs-log4j.properties config file - Done"))
        Logger.info("Creating symlinks - STARTED")
        Link("/usr/odp/current/hadoop-httpfs/libexec",to = "/usr/odp/current/hadoop-client/libexec")
        Logger.info("Creating symlinks - DONE")

    def httpfs_packages_fetcher(self):
        import params
        httpfs_packages = [
            '-httpfs',
            '-httpfs-server'
        ]
        httpfs_str = params.stack_version
        httpfs_str = 'hadoop_'+httpfs_str.replace('-','_').replace('.','_')
        for index in range(len(httpfs_packages)):
            httpfs_packages[index] = httpfs_str + httpfs_packages[index]
        return httpfs_packages

    def install(self, env):
        import params
        self.install_packages(env)
        self.httpfs_packages = self.httpfs_packages_fetcher()
        Logger.info("Installing HttpFS packages")
        if self.httpfs_packages is not None and len(self.httpfs_packages):
            for pack in self.httpfs_packages:
                Package(pack)

    def httpfs_additional_configs(self, env):
        secretFile = "/etc/hadoop/conf/httpfs-signature.secret"
        httpSecretFile = "/etc/security/http_secret"

        if not os.path.isfile(httpSecretFile):
            Execute("dd if=/dev/urandom of=/etc/security/http_secret bs=1024 count=1")
            Execute("chown hdfs:hadoop /etc/security/http_secret")
            Execute("chmod 440 /etc/security/http_secret")

        if os.path.exists(secretFile):
            if not os.path.islink(secretFile) or os.readlink(secretFile) != httpSecretFile:
                os.remove(secretFile)
                os.symlink(httpSecretFile, secretFile)
        else:
            os.symlink(httpSecretFile, secretFile)

    def stop(self, env):
        Logger.info("Stopping HttpFS service")
        Execute("/usr/odp/current/hadoop-httpfs/etc/rc.d/init.d/hadoop-httpfs stop")
        #Execute("/usr/bin/hdfs --daemon stop httpfs")
    def restart(self,env):
        self.stop(env)
        self.start(env)
    def start(self, env):
        import params,os
        self.configure(env)
        self.httpfs_additional_configs(env)
        Logger.info("Starting HttpFS service")
        ## Keeping below command for future use.
        ## Execute("su -s /bin/bash -c "/usr/odp/current/hadoop-httpfs/sbin/httpfs.sh start httpfs" httpfs")
        Execute("/usr/odp/current/hadoop-httpfs/etc/rc.d/init.d/hadoop-httpfs start")

    def status(self, env):
        Logger.info("Getting status of HttpFS service")
        try:
            Execute("/usr/odp/current/hadoop-httpfs/etc/rc.d/init.d/hadoop-httpfs status")
        except Fail:
            raise ComponentIsNotRunning()

if __name__ == "__main__":
    Httpfs_component().execute()
